@@ -1,11 +1,20 @@
+from typing import Final
+
 from docutils.parsers.rst.directives import register_directive
 from docutils.parsers.rst.roles import GenericRole, register_local_role
 
 from bpystubgen.directives import ClassDirective, CurrentModuleDirective, DataDirective, FunctionDirective, \
     ModuleDirective, PropertyDirective
-from bpystubgen.generator import generate
-from bpystubgen.nodes import AttributeRef, ClassRef, DataRef, PropertyRef, DocString, Function, FunctionRef, \
-    MethodRef, Module, ModuleRef, PythonRef, Reference
+from bpystubgen.nodes import AttributeRef, ClassRef, DataRef, DocString, Function, FunctionRef, MethodRef, Module, \
+    ModuleRef, PropertyRef, PythonRef, Reference
+
+lowercase_class_names: Final = {
+    "bpy.types.bpy_prop_collection",
+    "bpy.types.bpy_struct",
+    "bpy.types.wmOwnerID",
+    "bpy.types.wmOwnerIDs",
+    "bpy.types.wmTools"
+}
 
 register_directive("module", ModuleDirective)
 register_directive("data", DataDirective)
