@@ -10,7 +10,11 @@ _container_of_pattern: Final = re.compile(
     "(?P<data>[a-zA-Z]+))(?P<qualifier>[',.\\s].*)?$")
 
 _prop_collection_of_pattern: Final = re.compile(
-    "^:class:`bpy_prop_collection`\\sof\\s(?::class:`[~!]?(?P<reference>[^`]+)`|"
+    "^(?::class:`[~!]?(?P<base>[^`]+)`\\s+)?:class:`bpy_prop_collection`\\sof\\s(?::class:`[~!]?(?P<reference>[^`]+)`|"
+    "(?P<data>[a-zA-Z]+))(?P<qualifier>[',.\\s].*)?$")
+
+_exp_list_value_of_pattern: Final = re.compile(
+    "^:class:`[~!]?bge.types.EXP_ListValue`\\sof\\s(?::class:`[~!]?(?P<reference>[^`]+)`|"
     "(?P<data>[a-zA-Z]+))(?P<qualifier>[',.\\s].*)?$")
 
 _qualified_list_pattern: Final = re.compile(
@@ -31,6 +35,9 @@ _dictionary_pattern: Final = re.compile(
 _reference_type_pattern: Final = re.compile(
     "^:class:`[~!]?(?P<name>[a-zA-Z_0-9.\\s]+)(?:\\s*<(?P<target>[a-zA-Z_0-9.]+)>)?`(?:[,.\\s].*)?$")
 
+_union_pattern: Final = re.compile(
+    "^(?::class:`[~!]?[^`]+`|[a-zA-Z]+)(:?\\sor\\s(?::class:`[~!]?[^`]+`|[a-zA-Z]+))+(?:[,.\\s].*)?$")
+
 _known_types: Final = {
     "any": "typing.Any",
     "str": "str",
@@ -97,7 +104,9 @@ def parse_simple(text: str) -> Optional[str]:
 def parse_reference(text: str) -> Optional[str]:
     result = _reference_type_pattern.match(text)
 
-    return result.group("target") or result.group("name") if result else None
+    type_info = result.group("target") or result.group("name") if result else None
+
+    return "typing.Any" if type_info == "AnyType" else type_info
 
 
 def parse_qualified_list(text: str) -> Optional[str]:
@@ -140,6 +149,9 @@ def parse_prop_collection_of(text: str) -> Optional[str]:
     if not result:
         return None
 
+    base_type = result.group("base")
+
+    # noinspection DuplicatedCode
     data_type = result.group("data")
     qualifier = result.group("qualifier")
 
@@ -162,7 +174,51 @@ def parse_prop_collection_of(text: str) -> Optional[str]:
         elif qualifier.startswith("sequence"):
             data_type = f"typing.Sequence[{data_type}]"
 
-    return f"typing.Union[typing.Sequence[{data_type}], typing.Mapping[str, {data_type}]]"
+    types = [base_type] if base_type else []
+    types.append(f"typing.Sequence[{data_type}]")
+    types.append(f"typing.Mapping[str, {data_type}]")
+    types.append("bpy.types.bpy_prop_collection")
+
+    return f"typing.Union[{', '.join(types)}]"
+
+
+def parse_exp_list_value_of(text: str) -> Optional[str]:
+    result = _exp_list_value_of_pattern.match(text)
+
+    if not result:
+        return None
+
+    data_type = result.group("data")
+
+    # noinspection DuplicatedCode
+    qualifier = result.group("qualifier")
+
+    if data_type in _known_types:
+        data_type = _known_types[data_type]
+    else:
+        reference = result.group("reference")
+        data_type = parse_reference("".join((":class:`", reference, "`"))) if reference else None
+
+    if not data_type:
+        return None
+
+    if qualifier:
+        qualifier = qualifier.strip()
+
+        if qualifier.startswith("tuple"):
+            data_type = f"typing.Tuple[{data_type}, ...]"
+        elif qualifier.startswith("list"):
+            data_type = f"typing.List[{data_type}]"
+        elif qualifier.startswith("sequence"):
+            data_type = f"typing.Sequence[{data_type}]"
+
+    types = (
+        f"typing.Sequence[{data_type}]",
+        f"typing.Mapping[str, {data_type}]",
+        "bge.types.EXP_ListValue"
+    )
+
+    return f"typing.Union[{', '.join(types)}]"
 
 
 def parse_container_of(text: str) -> Optional[str]:
@@ -267,6 +323,25 @@ def parse_dictionary(text: str) -> Optional[str]:
     return f"typing.Dict[{key}, {value}]"
 
 
+def parse_union(text: str) -> Optional[str]:
+    result = _union_pattern.match(text)
+
+    if not result:
+        return None
+
+    expressions = tuple(text.replace("\n", "").split(" or "))
+
+    if "None" in expressions:
+        return None
+
+    parsed_types = tuple(map(parse_type, expressions))
+
+    if None in parsed_types or len(expressions) != len(set(parsed_types)):
+        return None
+
+    return f"typing.Union[{', '.join(parsed_types)}]"
+
+
 def parse_special_cases(text: str) -> Optional[str]:
     text = text.lower()
 
@@ -297,7 +372,9 @@ def parse_type(text: str) -> Optional[str]:
         parse_bracket_list,
         parse_container_of,
         parse_prop_collection_of,
+        parse_exp_list_value_of,
         parse_dictionary,
+        parse_union,
         parse_reference,
         parse_simple,
         parse_special_cases
